@@ -69,28 +69,7 @@ midas_u <- function(formula, data ,...) {
         ee <- NULL
     }
     else {
-        if(is.matrix(data)) data <- data.frame(data)
-        if(is.data.frame(data)) {
-            ee <- as.environment(as.list(data))
-        }
-        else {
-            if(is.list(data)) {
-                if(is.null(names(data))) names(data) <- rep("",length(data))
-                data <- mapply(function(x,nm){
-                    if(is.null(dim(x))) {
-                        x <- list(x)
-                        names(x) <- nm
-                        x
-                    } else {
-                        as.list(x)
-                    }
-                },data,names(data),SIMPLIFY=FALSE)
-                names(data) <- NULL
-                ee <- as.environment(do.call("c",data))
-            } else {
-                stop("Argument data must be a matrix, data.frame or a list")
-            }
-        }
+        ee <- data_to_env(data)
         parent.env(ee) <- parent.frame()
     }
     
@@ -101,6 +80,7 @@ midas_u <- function(formula, data ,...) {
     mf[[3L]] <- as.name("ee")   
    
     out <- eval(mf,Zenv)
+    out$Zenv <- Zenv
     class(out) <- c("midas_u",class(out))
     out
 }
@@ -323,7 +303,10 @@ midas_r.fit <- function(x) {
         if(x$user.gradient) {
             args$gr <- x$gradient
         }
-        opt <- do.call(function.opt,args)
+        opt <- try(do.call(function.opt,args),silent=TRUE)
+        if(class(opt)=="try-error") {
+            stop("The optimisation algorithm of MIDAS regression failed with the following message:\n", opt,"\nPlease try other starting values or a different optimisation function")
+        }
         par <- opt$par
         names(par) <- names(coef(x))
         x$convergence <- opt$convergence
@@ -348,7 +331,10 @@ midas_r.fit <- function(x) {
         y <- x$model[,1]
         args$formula <- formula(y~rhs(p))
         args$start <- list(p=x$start.opt)
-        opt <- do.call("nls",args)
+        opt <- try(do.call("nls",args),silent=TRUE)
+        if(class(opt)=="try-error") {
+            stop("The optimisation algorithm of MIDAS regression failed with the following message:\n", opt,"\nPlease try other starting values or a different optimisation function")
+        }
         par <- coef(opt)
         names(par) <- names(coef(x))
         x$convergence <- opt$convInfo$stopCode
